//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "MainFrm.h"
#include "KMTrackTypes.h"
#include "KMCustomItem.hpp"
#include <IniFiles.hpp>
#include <math.h>
#include "FXItem.hpp"
//---------------------------------------------------------------------------
#define _Round(theFVal) (floor(theFVal) + 0.5 > (theFVal)) ? floor(theFVal) : ceil(theFVal)
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "KMTimeLine"
#pragma link "KMCustomItem"
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
	: TForm(Owner)
{
	FRelTimeProcessing = true;
	//
	FOutputImg = new Graphics::TBitmap;
	OutputPaint->ControlStyle = OutputPaint->ControlStyle << csOpaque;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::InsertImageItem(void)
{
	TKMCustomItem* imgItem;
	TPicture* tmpPic;
	if (OpenPicDlg->Execute()) {
		tmpPic= new TPicture;
		try {
			tmpPic->LoadFromFile(OpenPicDlg->FileName);
			imgItem = new TKMCustomItem(KMTimeLinePanel1, 4, tmpPic->Graphic);
			imgItem->KMTrackTypeID = c_id_Images;
			if (KMTimeLinePanel1->SelectedTrack) KMTimeLinePanel1->InsertItem(imgItem, (TKMTrack*)NULL, c_ActTimeInit);
			else KMTimeLinePanel1->AppendItem(imgItem, (TKMTrack*)NULL);
		} __finally {
			delete tmpPic;
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TForm1::InsertFXItem(void)
{
	TFXItem* fxItem;
	fxItem = new TFXItem(KMTimeLinePanel1, 1);
	if (KMTimeLinePanel1->SelectedTrack) KMTimeLinePanel1->InsertItem(fxItem, (TKMTrack*)NULL, c_ActTimeInit);
	else KMTimeLinePanel1->AppendItem(fxItem, (TKMTrack*)NULL);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::DeleteItem(void)
{
	if (KMTimeLinePanel1->SelectedItem) {
		if (MessageDlg("Delete selected item?", mtWarning, TMsgDlgButtons() << mbOK << mbCancel, 0) == mrOk) {
			KMTimeLinePanel1->DeleteItem(KMTimeLinePanel1->SelectedItem);
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TForm1::StartPlay(void)
{
	KMTimeLinePanel1->StartProcess(tbr1Frame, FRelTimeProcessing, 0, 0);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::StartPlaySelection(void)
{
	TKMTimeLineBlock* block;
	//
	block = NULL;
	if (KMTimeLinePanel1->ForEachBlock(block)) {
		KMTimeLinePanel1->StartProcess(tbr1Frame, FRelTimeProcessing, block->TimeStart, block->TimeEnd);
	}
}
//---------------------------------------------------------------------------
void __fastcall TForm1::StopPlay(void)
{
	KMTimeLinePanel1->StopProcess();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::LoadContents(bool bFromIni)
{
	TIniFile* iniFile;
	AnsiString strFName;
	//
	if (bFromIni) {
		strFName = ChangeFileExt(Application->ExeName, ".ini");
		if (!FileExists(strFName)) return;
		iniFile = new TIniFile(strFName);
		try {
			strFName = iniFile->ReadString("KMTimeLine", "ContentsFile", "");
		} __finally {
			delete iniFile;
		}
		if ((strFName == "") || (!FileExists(strFName))) return;
	}
	else {
		if (!OpenDlg->Execute()) return;
		strFName = OpenDlg->FileName;
		if (!FileExists(strFName)) {
			strFName = ExtractFilePath(Application->ExeName) + ExtractFileName(strFName);
		}
		if (!FileExists(strFName)) {
			ShowMessage("KMTimeLine contents file: " + ExtractFileName(strFName) + " doesn`t exist!");
			return;
		}
		strFName = OpenDlg->FileName;
	}
	KMTimeLinePanel1->ClearContents();
	KMTimeLinePanel1->LoadContents(strFName);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::SaveContents(void)
{
	TIniFile* iniFile;
	AnsiString strFName;
	//
	if (!SaveDlg->Execute()) return;
	strFName = SaveDlg->FileName;
	KMTimeLinePanel1->SaveContents(strFName);
	iniFile = new TIniFile(ChangeFileExt(Application->ExeName, ".ini"));
	try {
		iniFile->WriteString("KMTimeLine", "ContentsFile", strFName);
	} __finally {
		delete iniFile;
	}
}
//---------------------------------------------------------------------------
void __fastcall TForm1::PrepareOutputImage(void)
{
	if (!FOutputImg) return;
	if (OutputPaint->Width > OutputPaint->Height) {
		FOutputImg->Height = OutputPaint->Height - 2;
		FOutputImg->Width = min(OutputPaint->Width - 2, (int)ceil(FOutputImg->Height*3/2));
	}
	else {
		FOutputImg->Width = OutputPaint->Width - 2;
		FOutputImg->Height = min(OutputPaint->Height - 2, (int)ceil(FOutputImg->Width/3*2));
	}
	OutputPaint->Repaint();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormDestroy(TObject *Sender)
{
	delete FOutputImg;
	FOutputImg = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::KMTimeLinePanel1ProcessFinished(TObject *Sender)
{
	ShowMessage("Finished!");
}
//---------------------------------------------------------------------------
void __fastcall TForm1::KMTimeLinePanel1ToolbarBtnClick(
      TKMLbSpeedButton *theSpButton, int Index)
{
	switch (Index) {
		case 0: {
			InsertImageItem();
			break;
		}
		case 1: {
			InsertFXItem();
			break;
		}
		case 2: {
			DeleteItem();
			break;
		}
		case 3: {
			StartPlay();
			break;
		}
		case 4: {
			StartPlaySelection();
			break;
		}
		case 5: {
			StopPlay();
			break;
		}
		case 6: {
			LoadContents();
			break;
		}
		case 7: {
			SaveContents();
			break;
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TForm1::KMTimeLinePanel1ProcessTrackItems(TObject *Sender,
      Extended theTimePos, const TKMTrackItemArr theItemArr,
			 const TKMTrackItemTimeArr theItemTime)
{
	static TKMTrackItem* fCurItem = NULL;
	//
	TFXItem*			 fxItem = NULL;
	TKMCustomItem* imgItem1 = NULL, *imgItem2 = NULL;
	bool				 	 bSwitch;
	int				 		 ind, fxInd = -1;
	//
	for (ind = 0; ind < theItemArr.Length; ind++) {
		if (theItemArr[ind]) {
			if (dynamic_cast<TFXItem*>(theItemArr[ind])) {
				fxInd = ind;
				fxItem = dynamic_cast<TFXItem*>(theItemArr[ind]);
			}
			else if (dynamic_cast<TKMCustomItem*>(theItemArr[ind])) {
				if (!imgItem1) imgItem1 = dynamic_cast<TKMCustomItem*>(theItemArr[ind]);
				else if (!imgItem2) imgItem2 = dynamic_cast<TKMCustomItem*>(theItemArr[ind]);
			}
		}
	}
	//
	if (fxItem) {
		bSwitch = (imgItem2 == fCurItem);
		if (!bSwitch) fxItem->ProcessEffect(FOutputImg, imgItem1, imgItem2, theItemTime[fxInd]);
		else fxItem->ProcessEffect(FOutputImg, imgItem2, imgItem1, theItemTime[fxInd]);
	}
	else if (imgItem1 || imgItem2) {
		if (!imgItem2) imgItem2 = imgItem1;
		FOutputImg->Canvas->StretchDraw(Rect(0, 0, FOutputImg->Width, FOutputImg->Height), imgItem2->Image);
		fCurItem = imgItem2;
	}
	else fCurItem = NULL;
	OutputPaint->Repaint();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Splitter1Moved(TObject *Sender)
{
	PrepareOutputImage();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormResize(TObject *Sender)
{
	PrepareOutputImage();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::OutputPaintPaint(TObject *Sender)
{
	int iXPos = ((OutputPaint->Width/2)  - (FOutputImg->Width/2));
	int iYPos = ((OutputPaint->Height/2) - (FOutputImg->Height/2));
	OutputPaint->Canvas->Draw(iXPos, iYPos, FOutputImg);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormShow(TObject *Sender)
{
	KMTimeLinePanel1->AppendBlock(1);
	LoadContents(true);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::InsertImageItem1Click(TObject *Sender)
{
	InsertImageItem();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::InsertFXItem1Click(TObject *Sender)
{
	InsertFXItem();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::DeleteItem1Click(TObject *Sender)
{
	DeleteItem();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Playentirecontents1Click(TObject *Sender)
{
	StartPlay();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Playmarkedcontents1Click(TObject *Sender)
{
	StartPlaySelection();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::RealTimeProcessing1Click(TObject *Sender)
{
	FRelTimeProcessing = !FRelTimeProcessing;
	((TMenuItem*)Sender)->Checked = FRelTimeProcessing;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Stopplaying1Click(TObject *Sender)
{
	StopPlay();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::LoadContents1Click(TObject *Sender)
{
	LoadContents();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::SaveContents1Click(TObject *Sender)
{
	SaveContents();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Exit1Click(TObject *Sender)
{
	Close();
}
//---------------------------------------------------------------------------
