unit MainFrm;

interface

uses
	 Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	 StdCtrls, ExtCtrls, KMTimeLine, KMTrackItem, ImgList, ExtDlgs, ComCtrls,
  Menus, ToolWin, KMLbSpeedButton;

type
	 TForm1 = class(TForm)
		 ImageList1: TImageList;
    OpenPicDlg: TOpenPictureDialog;
    KMTimeLinePanel1: TKMTimeLinePanel;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
		 Exit1: TMenuItem;
    Help1: TMenuItem;
		 About1: TMenuItem;
    Splitter1: TSplitter;
    OutputPaint: TPaintBox;
    OpenDlg: TOpenDialog;
    SaveDlg: TSaveDialog;
    InsertImageItem1: TMenuItem;
    InsertFXItem1: TMenuItem;
    N1: TMenuItem;
    DeleteItem1: TMenuItem;
    Playentirecontents1: TMenuItem;
    Playmarkedcontents1: TMenuItem;
    Stopplaying1: TMenuItem;
    N2: TMenuItem;
    LoadContents1: TMenuItem;
    SaveContents1: TMenuItem;
    RealTimeProcessing1: TMenuItem;
		 procedure FormCreate(Sender: TObject);
		 procedure KMTimeLinePanel1ProcessFinished(Sender: TObject);
		 procedure KMTimeLinePanel1ToolbarBtnClick(
			 theSpButton: TKMLbSpeedButton; Index: Integer);
    procedure KMTimeLinePanel1ProcessTrackItems(Sender: TObject;
			 theTimePos: Extended; const theItemArr: TKMTrackItemArr;
			 const theItemTime: TKMTrackItemTimeArr);
		 procedure Splitter1Moved(Sender: TObject);
		 procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure OutputPaintPaint(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure InsertImageItem1Click(Sender: TObject);
    procedure InsertFXItem1Click(Sender: TObject);
    procedure DeleteItem1Click(Sender: TObject);
    procedure Playentirecontents1Click(Sender: TObject);
    procedure Playmarkedcontents1Click(Sender: TObject);
    procedure RealTimeProcessing1Click(Sender: TObject);
    procedure Stopplaying1Click(Sender: TObject);
    procedure LoadContents1Click(Sender: TObject);
    procedure SaveContents1Click(Sender: TObject);
	 private
		 { Private declarations }
		 FOutputImg: TBitmap;
		 FRelTimeProcessing: Boolean;
		 //
		 procedure InsertImageItem;
		 procedure InsertFXItem;
		 procedure DeleteItem;
		 procedure StartPlay;
		 procedure StartPlaySelection;
		 procedure StopPlay;
		 procedure LoadContents(bFromIni: Boolean = false);
		 procedure SaveContents;
		 //
		 procedure PrepareOutputImage;
	 public
		 { Public declarations }
	 end;

var
	 Form1: TForm1;

implementation

{$R *.DFM}

uses math, IniFiles, KMCustomItem, KMTimeLinetypes, KMTimeLineBlock, FXItem;

{$INCLUDE KMTrackTypes.inc}

procedure TForm1.FormCreate(Sender: TObject);
begin
	FRelTimeProcessing := true;
	//
	FOutputImg := TBitmap.Create;
	OutputPaint.ControlStyle := OutputPaint.ControlStyle + [csOpaque];
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
	FOutputImg.Free;
	FOutputImg := nil;
end;

procedure TForm1.KMTimeLinePanel1ProcessFinished(Sender: TObject);
begin
	ShowMessage('Finished!');
end;

procedure TForm1.InsertFXItem;
var
	fxItem: TFXItem;
begin
	fxItem := TFXItem.CreateEx(KMTimeLinePanel1, 1);
	if (Assigned(KMTimeLinePanel1.SelectedTrack)) then
		KMTimeLinePanel1.InsertItem(fxItem)
	else
		KMTimeLinePanel1.AppendItem(fxItem);
end;

procedure TForm1.InsertImageItem;
var
	imgItem: TKMCustomItem;
	tmpPic: TPicture;
begin
	if (OpenPicDlg.Execute) then begin
		tmpPic:= TPicture.Create;
		try
			tmpPic.LoadFromFile(OpenPicDlg.FileName);
			imgItem :=  TKMCustomItem.CreateEx(KMTimeLinePanel1, 4, tmpPic.Graphic);
			imgItem.KMTrackTypeID := c_id_Images;
			if (Assigned(KMTimeLinePanel1.SelectedTrack)) then
				KMTimeLinePanel1.InsertItem(imgItem)
			else
				KMTimeLinePanel1.AppendItem(imgItem)
		finally
			tmpPic.Free;
		end;
	end;
end;

procedure TForm1.DeleteItem;
begin
	if (Assigned(KMTimeLinePanel1.SelectedItem)) then begin
		if (MessageDlg('Delete selected item?', mtWarning, mbOkCancel, 0) = mrOk) then begin
			KMTimeLinePanel1.DeleteItem(KMTimeLinePanel1.SelectedItem);
		end;
	end;
end;

procedure TForm1.StartPlay;
begin
	KMTimeLinePanel1.StartProcess(tbr1Frame, FRelTimeProcessing);
end;

procedure TForm1.StartPlaySelection;
var
	block: TKMTimeLineBlock;
begin
	block := nil;
	if (KMTimeLinePanel1.ForEachBlock(block)) then begin
		KMTimeLinePanel1.StartProcess(tbr1Frame, FRelTimeProcessing, block.TimeStart, block.TimeEnd);
	end;
end;

procedure TForm1.StopPlay;
begin
	KMTimeLinePanel1.StopProcess;
end;

procedure TForm1.LoadContents(bFromIni: Boolean = false);
var
	iniFile: TIniFile;
	strFName: String;
begin
	if (bFromIni) then begin
		strFName := ChangeFileExt(Application.ExeName, '.ini');
		if (not FileExists(strFName)) then Exit;
		iniFile := TIniFile.Create(strFName);
	  try
	  	strFName := iniFile.ReadString('KMTimeLine', 'ContentsFile', '');
	  finally
	  	iniFile.Free;
		end;
		if ((strFName = '') or (not FileExists(strFName))) then Exit;
	end
	else begin
		if (not OpenDlg.Execute) then Exit;
		strFName := OpenDlg.FileName;
		if (not FileExists(strFName)) then begin
			strFName := ExtractFilePath(Application.ExeName) + ExtractFileName(strFName);
		end;
		if (not FileExists(strFName)) then begin
	  	ShowMessage('KMTimeLine contents file: ' + ExtractFileName(strFName) + ' doesn`t exist!');
	  	Exit;
	  end;
		strFName := OpenDlg.FileName;
	end;
	KMTimeLinePanel1.ClearContents;
	KMTimeLinePanel1.LoadContents(strFName);
end;

procedure TForm1.SaveContents;
var
	iniFile: TIniFile;
	strFName: String;
begin
	if (not SaveDlg.Execute) then Exit;
	strFName := SaveDlg.FileName;
	KMTimeLinePanel1.SaveContents(strFName);
	iniFile := TIniFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
	try
		iniFile.WriteString('KMTimeLine', 'ContentsFile', strFName);
	finally
		iniFile.Free;
	end;
end;

procedure TForm1.KMTimeLinePanel1ToolbarBtnClick(theSpButton: TKMLbSpeedButton; Index: Integer);
begin
	case Index of 
		0: begin
			InsertImageItem;
		end;
		1: begin
			InsertFXItem;
		end;
		2: begin
			DeleteItem;
		end;
		3: begin
			StartPlay;
		end;
		4: begin
			StartPlaySelection;
		end;
		5: begin
			StopPlay;
		end;
		6: begin
			LoadContents;
		end;
		7: begin
			SaveContents;
		end;
	end;
end;

procedure TForm1.KMTimeLinePanel1ProcessTrackItems(Sender: TObject;
																										theTimePos: Extended; 
																										const theItemArr: TKMTrackItemArr;
																										const theItemTime: TKMTrackItemTimeArr);
const
	fCurItem: TKMTrackItem = nil;
var
	fxItem: TFXItem;
	imgItem1, imgItem2: TKMCustomItem;
	bSwitch: Boolean;
	ind, fxInd: Integer;
begin
	fxItem := nil;
	imgItem1 := nil;
	imgItem2 := nil;
	fxInd := -1;
	//
	for ind := 0 to Length(theItemArr)-1 do begin
		if (Assigned(theItemArr[ind])) then begin
			if (theItemArr[ind] is TFXItem) then begin
				fxInd := ind;
				fxItem := (theItemArr[ind] as TFXItem);
			end
			else if (theItemArr[ind] is TKMCustomItem) then begin
				if (not Assigned(imgItem1)) then imgItem1 := (theItemArr[ind] as TKMCustomItem)
				else if (not Assigned(imgItem2)) then imgItem2 := (theItemArr[ind] as TKMCustomItem);
			end;
		end;
	end;
	//
	if (Assigned(fxItem)) then begin
		bSwitch := (imgItem2 = fCurItem);
		if (not bSwitch) then fxItem.ProcessEffect(FOutputImg, imgItem1, imgItem2, theItemTime[fxInd])
		else fxItem.ProcessEffect(FOutputImg, imgItem2, imgItem1, theItemTime[fxInd]);
	end
	else if (Assigned(imgItem1) or Assigned(imgItem2)) then begin
		if (not Assigned(imgItem2)) then imgItem2 := imgItem1;
		FOutputImg.Canvas.StretchDraw(Rect(0, 0, FOutputImg.Width, FOutputImg.Height), imgItem2.Image);
		fCurItem := imgItem2;
	end
	else fCurItem := nil;
	OutputPaint.Repaint;
end;

procedure TForm1.Splitter1Moved(Sender: TObject);
begin
	PrepareOutputImage;
end;

procedure TForm1.PrepareOutputImage;
begin
	if (not Assigned(FOutputImg)) then Exit;
	if (OutputPaint.Width > OutputPaint.Height) then begin
		FOutputImg.Height := OutputPaint.Height - 2;
		FOutputImg.Width := min(OutputPaint.Width - 2, ceil(FOutputImg.Height*3/2));
	end
	else begin
		FOutputImg.Width := OutputPaint.Width - 2;
		FOutputImg.Height := min(OutputPaint.Height - 2, ceil(FOutputImg.Width/3*2));
	end;
	OutputPaint.Repaint;
end;

procedure TForm1.FormResize(Sender: TObject);
begin
	PrepareOutputImage;
end;

procedure TForm1.OutputPaintPaint(Sender: TObject);
begin
	OutputPaint.Canvas.Draw(Round(OutputPaint.Width/2) - Round(FOutputImg.Width/2), 
													Round(OutputPaint.Height/2) - Round(FOutputImg.Height/2),
													FOutputImg);
end;

procedure TForm1.FormShow(Sender: TObject);
begin
	KMTimeLinePanel1.AppendBlock(1);
	LoadContents(true);
end;

procedure TForm1.InsertImageItem1Click(Sender: TObject);
begin
	InsertImageItem;
end;

procedure TForm1.InsertFXItem1Click(Sender: TObject);
begin
	InsertFXItem;
end;

procedure TForm1.DeleteItem1Click(Sender: TObject);
begin
	DeleteItem;
end;

procedure TForm1.Playentirecontents1Click(Sender: TObject);
begin
	StartPlay;
end;

procedure TForm1.Playmarkedcontents1Click(Sender: TObject);
begin
	StartPlaySelection;
end;

procedure TForm1.RealTimeProcessing1Click(Sender: TObject);
begin
	FRelTimeProcessing := not FRelTimeProcessing;
	(Sender as TMenuItem).Checked := FRelTimeProcessing;
end;

procedure TForm1.Stopplaying1Click(Sender: TObject);
begin
	StopPlay;
end;

procedure TForm1.LoadContents1Click(Sender: TObject);
begin
	LoadContents;
end;

procedure TForm1.SaveContents1Click(Sender: TObject);
begin
	SaveContents;
end;

end.

